@@ -322,10 +322,12 @@ def get_allowed_origins_legacy():
     allowed_origins = []
     for s in apt_pkg.config.value_list("Unattended-Upgrade::Allowed-Origins"):
         # if there is a ":" use that as seperator, else use spaces
-        if ":" in s:
-            (distro_id, distro_codename) = s.split(':')
+        if re.findall(r'(?<!\\):', s):
+            (distro_id, distro_codename) = re.split(r'(?<!\\):', s)
         else:
             (distro_id, distro_codename) = s.split()
+        # unescape "\:" back to ":"
+        distro_id = re.sub(r'\\:', ':', distro_id)
         # escape "," (see LP: #824856) - i wonder if there is a simpler way?
         distro_id = re.sub(r'([^\\]),', r'\1\\,', distro_id)
         distro_codename = re.sub(r'([^\\]),', r'\1\\,', distro_codename)
@@ -450,9 +452,9 @@ def upgrade_in_minimal_steps(cache, pkgs_to_upgrade, blacklist,
                 return True
             pkg = cache[pkgname]
             if pkg.is_upgradable:
-                pkg.mark_upgrade()
+                pkg.mark_upgrade(from_user=not pkg.is_auto_installed)
             elif not pkg.is_installed:
-                pkg.mark_install()
+                pkg.mark_install(from_user=False)
             else:
                 continue
             # double check that we are not running into side effects like
@@ -550,9 +552,13 @@ def is_pkgname_in_whitelist(pkgname, whitelist):
     return False
 
 
-def check_changes_for_sanity(cache, allowed_origins, blacklist, whitelist):
+def check_changes_for_sanity(cache, allowed_origins, blacklist, whitelist,
+                             desired_pkg=None):
     if cache._depcache.broken_count != 0:
         return False
+    # If there are no packages to be installed they were kept back
+    if cache.install_count == 0:
+        return False
     for pkg in cache:
         if pkg.marked_delete:
             logging.debug("pkg '%s' now marked delete" % pkg.name)
@@ -594,6 +600,9 @@ def check_changes_for_sanity(cache, allowed_origins, blacklist, whitelist):
                 logging.debug("pkg '%s' requires app-restart, not safe to "
                               "upgrade unattended")
                 return False
+    # check that the package we want to upgrade is in the change set
+    if desired_pkg and desired_pkg not in cache.get_changes():
+        return False
     return True
 
 
@@ -658,11 +667,11 @@ def conffile_prompt(destFile, prefix=""):
             continue
         # show what we do
         logging.debug("conffile line: '%s'", line)
-        l = line.split()
-        conf_file = l[0]
-        md5 = l[1]
-        if len(l) > 2:
-            obs = l[2]
+        li = line.split()
+        conf_file = li[0]
+        md5 = li[1]
+        if len(li) > 2:
+            obs = li[2]
         else:
             obs = None
         # ignore if conffile is obsolete
@@ -734,7 +743,7 @@ def rewind_cache(cache, pkgs_to_upgrade):
     """ set the cache back to the state with packages_to_upgrade """
     cache.clear()
     for pkg2 in pkgs_to_upgrade:
-        pkg2.mark_install()
+        pkg2.mark_install(from_user=not pkg2.is_auto_installed)
     if cache.broken_count > 0:
         raise AssertionError("rewind_cache created a broken cache")
 
@@ -1053,9 +1062,10 @@ def try_to_upgrade(pkg, pkgs_to_upgrade, pkgs_kept_back, cache,
                    allowed_origins, blacklisted_pkgs, whitelisted_pkgs):
 
     try:
-        pkg.mark_upgrade()
+        pkg.mark_upgrade(from_user=not pkg.is_auto_installed)
         if check_changes_for_sanity(cache, allowed_origins,
-                                    blacklisted_pkgs, whitelisted_pkgs):
+                                    blacklisted_pkgs, whitelisted_pkgs,
+                                    pkg):
             # add to packages to upgrade
             pkgs_to_upgrade.append(pkg)
             # re-eval pkgs_kept_back as the resolver may fail to
@@ -1129,13 +1139,15 @@ def get_auto_removable(cache):
                 if pkg.is_auto_removable])
 
 
-def do_auto_remove(cache, auto_removable, logfile_dpkg, verbose=False):
+def do_auto_remove(cache, auto_removable, logfile_dpkg,
+                   verbose=False, dry_run=False):
     if not auto_removable:
         return True
 
     for pkgname in auto_removable:
-        logging.debug("marking %s for remove" % pkgname)
-        cache[pkgname].mark_delete()
+        if not dry_run:
+            logging.debug("marking %s for remove" % pkgname)
+            cache[pkgname].mark_delete()
     logging.info(_("Packages that are auto removed: '%s'"),
                  " ".join(sorted(auto_removable)))
 
@@ -1281,7 +1293,7 @@ def main(options, rootdir=""):
     actiongroup = apt_pkg.ActionGroup(cache._depcache)
     actiongroup  # pyflakes
 
-    # find out about the packages that are upgradable (in a allowed_origin)
+    # find out about the packages that are upgradable (in an allowed_origin)
     pkgs_to_upgrade, pkgs_kept_back = calculate_upgradable_pkgs(
         cache, options, allowed_origins, blacklisted_pkgs, whitelisted_pkgs)
     pkgs_to_upgrade.sort(key=lambda p: p.name)
@@ -1308,6 +1320,9 @@ def main(options, rootdir=""):
     res = fetcher.run()
     logging.debug("fetch.run() result: %s", res)
 
+    if options.download_only:
+        return
+
     if dpkg_conffile_prompt():
         # now check the downloaded debs for conffile conflicts and build
         # a blacklist
@@ -1362,7 +1377,7 @@ def main(options, rootdir=""):
             for pkg in old_pkgs_to_upgrade:
                 logging.debug("Checking the black and whitelist: %s" %
                               (pkg.name))
-                pkg.mark_upgrade()
+                pkg.mark_upgrade(from_user=not pkg.is_auto_installed)
                 if check_changes_for_sanity(cache,
                                             allowed_origins,
                                             blacklisted_pkgs,
@@ -1374,7 +1389,7 @@ def main(options, rootdir=""):
                     logging.info(_("package '%s' not upgraded"), pkg.name)
                     cache.clear()
                     for pkg2 in pkgs_to_upgrade:
-                        pkg2.mark_upgrade()
+                        pkg2.mark_upgrade(from_user=not pkg2.is_auto_installed)
     else:
         logging.debug("dpkg is configured not to cause conffile prompts")
 
@@ -1396,7 +1411,9 @@ def main(options, rootdir=""):
         write_stamp_file()
         # check if we couldn't reboot on previous run because
         # a user was logged-in at this time
-        reboot_if_requested_and_needed()
+        # never reboot during a dry run
+        if not options.dry_run:
+            reboot_if_requested_and_needed()
         return
 
     # check if its configured for install on shutdown, if so, the
@@ -1453,7 +1470,8 @@ def main(options, rootdir=""):
         auto_removals = get_auto_removable(cache)
         pkg_install_success &= do_auto_remove(
             cache, auto_removals, logfile_dpkg,
-            options.verbose or options.debug)
+            options.verbose or options.debug,
+            options.dry_run)
     # the user wants *only new* auto-removals to be removed
     elif apt_pkg.config.find_b(
             "Unattended-Upgrade::Remove-New-Unused-Dependencies", True):
@@ -1462,7 +1480,8 @@ def main(options, rootdir=""):
         auto_removals = new_pending_autoremovals - previous_autoremovals
         pkg_install_success &= do_auto_remove(
             cache, auto_removals, logfile_dpkg,
-            options.verbose or options.debug)
+            options.verbose or options.debug,
+            options.dry_run)
 
     logging.debug("InstCount=%i DelCount=%i BrokenCount=%i"
                   % (cache._depcache.inst_count,
@@ -1514,6 +1533,9 @@ if __name__ == "__main__":
     parser.add_option("", "--dry-run",
                       action="store_true", default=False,
                       help=_("Simulation, download but do not install"))
+    parser.add_option("", "--download-only",
+                      action="store_true", default=False,
+                      help=_("Only download, do not even try to install."))
     parser.add_option("", "--minimal-upgrade-steps",
                       action="store_true", default=False,
                       help=_("Upgrade in minimal steps (and allow "
